"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var definitions_interface_1 = require("../../definitions.interface");
var di_1 = require("../../di");
var util_1 = require("../../util");
var DateTimeFieldHelper = /** @class */ (function () {
    function DateTimeFieldHelper() {
    }
    DateTimeFieldHelper_1 = DateTimeFieldHelper;
    /**
     * @stable [25.08.2018]
     * @param {TEntity} changes
     * @param {(entity: TEntity) => string} fromDateResolver
     * @param {(entity: TEntity) => string} toDateResolver
     * @param {number} monthAgo
     * @returns {IFromDateToDateEntity}
     */
    DateTimeFieldHelper.prototype.buildLastPeriodFromChanges = function (changes, fromDateResolver, toDateResolver, monthAgo) {
        return this.buildLastPeriod({ changes: changes }, fromDateResolver, toDateResolver, monthAgo);
    };
    /**
     * @stable [25.08.2018]
     * @param {IApiEntity<TEntity extends IEntity>} apiEntity
     * @param {(entity: TEntity) => string} fromDateResolver
     * @param {(entity: TEntity) => string} toDateResolver
     * @param {number} monthAgo
     * @returns {IFromDateToDateEntity}
     */
    DateTimeFieldHelper.prototype.buildLastPeriod = function (apiEntity, fromDateResolver, toDateResolver, monthAgo) {
        if (monthAgo === void 0) { monthAgo = 0; }
        var lastDayOfLastMonth = this.dc.getLastDayOfMonth(monthAgo);
        var firstDayOfLastMonth = this.dc.getFirstDayOfMonth(monthAgo);
        var currentDate = this.dc.getCurrentDate();
        return {
            fromDate: this.dc.fromDateTimeToDate(this.toDateTime(apiEntity, fromDateResolver) || firstDayOfLastMonth),
            toDate: this.dc.fromDateTimeToDate(this.toDateTime(apiEntity, toDateResolver) ||
                (lastDayOfLastMonth.getTime() > currentDate.getTime() ? currentDate : lastDayOfLastMonth)),
        };
    };
    // TODO duplication
    DateTimeFieldHelper.prototype.buildLastDateTimePeriod = function (apiEntity, fromDateResolver, toDateResolver, monthAgo) {
        if (monthAgo === void 0) { monthAgo = 0; }
        var lastDayOfLastMonth = this.dc.getLastDayOfMonth(monthAgo);
        var firstDayOfLastMonth = this.dc.getFirstDayOfMonth(monthAgo);
        var currentDate = this.dc.getCurrentDate();
        return {
            fromDate: this.dc.formatDateTimeFromDateTime(this.toDateTime(apiEntity, fromDateResolver) || firstDayOfLastMonth),
            toDate: this.dc.formatDateTimeFromDateTime(this.toDateTime(apiEntity, toDateResolver) ||
                (lastDayOfLastMonth.getTime() > currentDate.getTime() ? currentDate : lastDayOfLastMonth)),
        };
    };
    DateTimeFieldHelper.prototype.buildLastDateTimePeriodFromChanges = function (changes, fromDateResolver, toDateResolver, monthAgo) {
        return this.buildLastDateTimePeriod({ changes: changes }, fromDateResolver, toDateResolver, monthAgo);
    };
    DateTimeFieldHelper.prototype.buildDateTimeFromField = function (apiEntity, returnOriginalValueIfNoChanges) {
        if (returnOriginalValueIfNoChanges === void 0) { returnOriginalValueIfNoChanges = false; }
        return this.toDateTime(apiEntity, function (source) { return source.date; }, function (source) { return source.time; }, returnOriginalValueIfNoChanges, definitions_interface_1.DEFAULT_TIME_FROM);
    };
    DateTimeFieldHelper.prototype.buildDateTimeSinceField = function (apiEntity, returnOriginalValueIfNoChanges) {
        if (returnOriginalValueIfNoChanges === void 0) { returnOriginalValueIfNoChanges = false; }
        return this.toDateTime(apiEntity, function (source) { return source.fromDate; }, function (source) { return source.fromTime; }, returnOriginalValueIfNoChanges, definitions_interface_1.DEFAULT_TIME_FROM);
    };
    DateTimeFieldHelper.prototype.buildDateTimeTillField = function (apiEntity, returnOriginalValueIfNoChanges) {
        if (returnOriginalValueIfNoChanges === void 0) { returnOriginalValueIfNoChanges = false; }
        return this.toDateTime(apiEntity, function (source) { return source.toDate; }, function (source) { return source.toTime; }, returnOriginalValueIfNoChanges, definitions_interface_1.DEFAULT_TIME_TO);
    };
    DateTimeFieldHelper.prototype.joinDateTimeFields = function (apiEntity, returnOriginalValueIfNoChanges) {
        if (returnOriginalValueIfNoChanges === void 0) { returnOriginalValueIfNoChanges = false; }
        return __assign({}, this.composeDateTimeSinceField(apiEntity, returnOriginalValueIfNoChanges), this.composeDateTimeTillField(apiEntity, returnOriginalValueIfNoChanges));
    };
    DateTimeFieldHelper.prototype.composeDateTimeSinceField = function (apiEntity, returnOriginalValueIfNoChanges) {
        if (returnOriginalValueIfNoChanges === void 0) { returnOriginalValueIfNoChanges = false; }
        return {
            fromDate: this.buildDateTimeSinceField(apiEntity, returnOriginalValueIfNoChanges),
        };
    };
    DateTimeFieldHelper.prototype.composeDateTimeTillField = function (apiEntity, returnOriginalValueIfNoChanges) {
        if (returnOriginalValueIfNoChanges === void 0) { returnOriginalValueIfNoChanges = false; }
        return {
            toDate: this.buildDateTimeTillField(apiEntity, returnOriginalValueIfNoChanges),
        };
    };
    /**
     * @stable [02.06.2018]
     * @param {TEntity} entity
     * @param {Date} defaultFromDate
     * @param {Date} defaultToDate
     * @returns {IFromDateToDateEntity}
     */
    DateTimeFieldHelper.prototype.composeDateTimeRangeFields = function (entity, defaultFromDate, defaultToDate) {
        if (defaultFromDate === void 0) { defaultFromDate = this.dc.get30DaysAgo(); }
        if (defaultToDate === void 0) { defaultToDate = this.dc.getCurrentDate(); }
        return this.buildDateTimeRangeFields(entity, function (source) { return source.fromDate; }, function (source) { return source.toDate; }, function (source) { return source.fromTime; }, function (source) { return source.toTime; }, definitions_interface_1.FROM_DATE_FIELD_NAME, definitions_interface_1.TO_DATE_FIELD_NAME, defaultFromDate, defaultToDate);
    };
    DateTimeFieldHelper.prototype.buildDateTimeRangeFields = function (entity, fromDateResolver, toDateResolver, fromTimeResolver, toTimeResolver, fromDateFieldName, toDateFieldName, defaultFromDate, defaultToDate) {
        var _a;
        return _a = {},
            _a[fromDateFieldName] = this.dc.fromStartUiDateTimeToDateTime(fromDateResolver(entity) || this.dc.fromDateToUiDate(defaultFromDate), fromTimeResolver(entity)),
            _a[toDateFieldName] = this.dc.fromEndUiDateTimeToDateTime(toDateResolver(entity) || this.dc.fromDateToUiDate(defaultToDate), toTimeResolver(entity)),
            _a;
    };
    DateTimeFieldHelper.prototype.splitToDateTimeRangeFields = function (entity) {
        return __assign({}, this.splitToDateTimeSinceFields(entity), this.splitToDateTimeTillFields(entity));
    };
    DateTimeFieldHelper.prototype.splitToDateTimeSinceFields = function (entity) {
        return this.splitToDateTimeFields(entity, definitions_interface_1.FROM_DATE_FIELD_NAME, definitions_interface_1.FROM_TIME_FIELD_NAME, function (source) { return source.fromDate; });
    };
    DateTimeFieldHelper.prototype.splitToDateTimeTillFields = function (entity) {
        return this.splitToDateTimeFields(entity, definitions_interface_1.TO_DATE_FIELD_NAME, definitions_interface_1.TO_TIME_FIELD_NAME, function (source) { return source.toDate; });
    };
    DateTimeFieldHelper.prototype.splitToDateTimeBasicFields = function (entity) {
        return this.splitToDateAndTimeBasicFields(entity, function (source) { return source.date; });
    };
    DateTimeFieldHelper.prototype.splitToDateAndTimeBasicFields = function (entity, dateResolver) {
        return this.splitToDateTimeFields(entity, definitions_interface_1.DATE_FIELD_NAME, definitions_interface_1.TIME_FIELD_NAME, dateResolver);
    };
    /**
     * @deprecated
     */
    DateTimeFieldHelper.prototype.splitToDateTimeFields = function (entity, dateFieldName, timeFieldName, dateResolver) {
        var _this = this;
        var _a;
        var date = dateResolver(entity);
        return _a = {},
            _a[timeFieldName] = util_1.orUndef(date, function () { return _this.dc.fromDateTimeToTime(date); }),
            _a[dateFieldName] = util_1.orUndef(date, function () { return _this.dc.fromDateTimeToDate(date); }),
            _a;
    };
    DateTimeFieldHelper.prototype.toDateTime = function (apiEntity, dateResolver, timeResolver, returnOriginalValueIfNoChanges, defaultTime) {
        if (returnOriginalValueIfNoChanges === void 0) { returnOriginalValueIfNoChanges = false; }
        var returnedDate;
        var returnedTime;
        var entityChanges = apiEntity.changes;
        var entity = apiEntity.entity;
        var changedDate = dateResolver(entityChanges);
        var changedTime = util_1.orUndef(util_1.isFn(timeResolver), function () { return timeResolver(entityChanges); });
        if (util_1.isUndef(changedDate) && util_1.isUndef(changedTime) && !returnOriginalValueIfNoChanges) {
            // The are no changes at all and we don't want to return the original value
            return definitions_interface_1.UNDEF;
        }
        if (util_1.isNull(changedDate)) {
            // Reset a date => send null to the server
            return null;
        }
        returnedDate = changedDate;
        returnedTime = changedTime;
        var originalDate = entity && dateResolver(entity);
        var originalTime = entity && (util_1.orUndef(util_1.isFn(timeResolver), function () { return timeResolver(entity); }));
        if (util_1.isNull(changedTime)) {
            // Reset a time
            if (R.isNil(defaultTime)) {
                // But default time is not provided
                returnedTime = originalTime;
            }
            else {
                returnedTime = defaultTime;
            }
        }
        else {
            returnedTime = changedTime || originalTime || defaultTime;
        }
        return this.dc.fromUiDateTimeToDateTime(returnedDate || originalDate, returnedTime);
    };
    var DateTimeFieldHelper_1;
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.DateConverter),
        __metadata("design:type", Object)
    ], DateTimeFieldHelper.prototype, "dc", void 0);
    DateTimeFieldHelper = DateTimeFieldHelper_1 = __decorate([
        di_1.provideInSingleton(DateTimeFieldHelper_1)
    ], DateTimeFieldHelper);
    return DateTimeFieldHelper;
}());
exports.DateTimeFieldHelper = DateTimeFieldHelper;
//# sourceMappingURL=date-time-field-helper.service.js.map