"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts_smart_logger_1 = require("ts-smart-logger");
var util_1 = require("../../util");
var di_1 = require("../../di");
var application_interface_1 = require("../application/application.interface");
var stack_interface_1 = require("../../store/stack/stack.interface");
var router_interface_1 = require("../../router/router.interface");
var universal_connector_interface_1 = require("./universal-connector.interface");
var universal_connector_factory_1 = require("./universal-connector.factory");
var universal_connector_action_builder_1 = require("./universal-connector-action.builder");
var logger = ts_smart_logger_1.LoggerFactory.makeLogger('universal-connector.decorator');
/**
 * @stable - 23.04.2018
 * @param {IBasicConnectorConfiguration<TStoreEntity>} config
 * @returns {(target: IContainerClassEntity) => void}
 */
exports.basicConnector = function (config) {
    return function (target) {
        if (config.callback) {
            config.callback(target);
        }
        var sectionName = target.defaultProps && target.defaultProps.sectionName;
        if (sectionName) {
            Reflect.set(target, universal_connector_interface_1.CONNECTOR_SECTION_FIELD, sectionName);
            var sectionName0_1 = sectionName;
            application_interface_1.APPLICATION_SECTIONS.set(sectionName0_1, config);
            var proto = target.prototype;
            proto.componentWillUnmount = util_1.sequence(proto.componentWillUnmount || util_1.noop, function () {
                var store = di_1.staticInjector(di_1.DI_TYPES.Store);
                store.dispatch({ type: stack_interface_1.STACK_POP_ACTION_TYPE, data: sectionName0_1 });
                store.dispatch({ type: universal_connector_action_builder_1.UniversalConnectorActionBuilder.buildDestroyActionType(sectionName0_1) });
                logger.debug("[$basicConnector][componentWillUnmount] Section: " + sectionName0_1);
            });
            proto.componentDidMount = util_1.sequence(proto.componentDidMount || util_1.noop, function () {
                var store = di_1.staticInjector(di_1.DI_TYPES.Store);
                store.dispatch({ type: stack_interface_1.STACK_PUSH_ACTION_TYPE, data: sectionName0_1 });
                store.dispatch({ type: universal_connector_action_builder_1.UniversalConnectorActionBuilder.buildInitActionType(sectionName0_1) });
                logger.debug("[$basicConnector][componentDidMount] Section: " + sectionName0_1);
            });
        }
        else {
            logger.warn("[$basicConnector] The sectionName props is not defined for " + (target.name ||
                target) + ". The init and destroy actions are disabled.");
        }
        router_interface_1.DYNAMIC_ROUTES.set(universal_connector_factory_1.universalConnectorFactory.apply(void 0, [target].concat(config.mappers)), config);
    };
};
/**
 * @stable - 23.04.2018
 * @param {IConnectorConfiguration<TStoreEntity extends IUniversalApplicationStoreEntity, TAccessConfig>} config
 * @returns {(target: IContainerClassEntity) => void}
 */
exports.connector = function (config) { return exports.basicConnector(config); };
//# sourceMappingURL=universal-connector.decorator.js.map