"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var ts_smart_logger_1 = require("ts-smart-logger");
var util_1 = require("../../../util");
var textfield_1 = require("../../field/textfield");
var menu_1 = require("../../menu");
var field_interface_1 = require("../../field/field/field.interface");
var BasicSelect = /** @class */ (function (_super) {
    __extends(BasicSelect, _super);
    /**
     * @stable [15.09.2018]
     * @param {TProps} props
     */
    function BasicSelect(props) {
        var _this = _super.call(this, props) || this;
        _this.onSelect = _this.onSelect.bind(_this);
        _this.openMenu = _this.openMenu.bind(_this);
        if (props.expandActionRendered !== false) {
            _this.defaultActions = R.insert(0, { type: 'expand_more', onClick: _this.openMenu }, _this.defaultActions);
        }
        return _this;
    }
    /**
     * @stable [20.08.2018]
     * @param {Readonly<TProps extends IBasicSelectProps>} prevProps
     * @param {Readonly<TState extends IBasicSelectState>} prevState
     */
    BasicSelect.prototype.componentDidUpdate = function (prevProps, prevState) {
        _super.prototype.componentDidUpdate.call(this, prevProps, prevState);
        var props = this.props;
        if (this.state.needToOpenMenu && !R.isNil(props.options)) {
            this.showMenu();
            this.setState({ needToOpenMenu: false });
            if (props.onLoadDictionary) {
                props.onLoadDictionary(this.toFilteredOptions());
            }
        }
    };
    BasicSelect.prototype.onKeyDown = function (event) {
        _super.prototype.onKeyDown.call(this, event);
        util_1.cancelEvent(event);
    };
    BasicSelect.prototype.onKeyBackspace = function (event) {
        _super.prototype.onKeyBackspace.call(this, event);
        this.clearValue();
    };
    BasicSelect.prototype.onKeyEnter = function (event) {
        _super.prototype.onKeyEnter.call(this, event);
        this.openMenu(event);
    };
    BasicSelect.prototype.onKeyEscape = function (event) {
        _super.prototype.onKeyEscape.call(this, event);
        if (this.menu.isOpen()) {
            this.hideMenu();
        }
    };
    /**
     * @stable [06.10.2018]
     */
    BasicSelect.prototype.clearValue = function () {
        _super.prototype.clearValue.call(this);
        this.setState({ displayValue: null });
    };
    /**
     * @stable [18.06.2018]
     * @returns {boolean}
     */
    BasicSelect.prototype.inProgress = function () {
        return _super.prototype.inProgress.call(this) || !!this.state.needToOpenMenu;
    };
    BasicSelect.prototype.onClick = function (event) {
        _super.prototype.onClick.call(this, event);
        this.openMenu(event);
    };
    /**
     * @stable [01.06.2018]
     * @returns {JSX.Element}
     */
    BasicSelect.prototype.getInputAttachmentElement = function () {
        var _this = this;
        var props = this.props;
        return (React.createElement(menu_1.Menu, __assign({ ref: 'menu', width: function () { return util_1.getWidth(_this.getSelf()); }, options: this.toFilteredOptions(), onSelect: this.onSelect }, props.menuConfiguration)));
    };
    /**
     * @stable [06.10.2018]
     * @returns {string}
     */
    BasicSelect.prototype.getInputElementWrapperClassName = function () {
        return util_1.toClassName(_super.prototype.getInputElementWrapperClassName.call(this), 'rac-flex-column' // popup menu
        );
    };
    /**
     * @stable [20.08.2018]
     * @returns {ISelectOptionEntity[]}
     */
    BasicSelect.prototype.toFilteredOptions = function () {
        return this.options;
    };
    /**
     * @stable [29.06.2018]
     * @returns {AnyT}
     */
    BasicSelect.prototype.getEmptyValue = function () {
        var props = this.props;
        return util_1.isDef(props.emptyValue) ? props.emptyValue : this.settings.entityEmptyId;
    };
    /**
     * @stable [20.08.2018]
     * @param {ISelectOptionEntity} option
     */
    BasicSelect.prototype.onSelect = function (option) {
        this.onChangeManually(option.value);
        /**
         * We need to save a label because dictionary data may be refreshed
         * TODO Need to save an internal state in storage
         */
        this.setState({ displayValue: option.label || option.value });
        if (this.props.onSelect) {
            this.props.onSelect(option);
        }
    };
    /**
     * @stable [19.12.2018]
     * @param {AnyT} value
     * @returns {EntityIdT}
     */
    BasicSelect.prototype.toDisplayValue = function (value) {
        if (this.inProgress()) {
            return field_interface_1.FIELD_DISPLAY_EMPTY_VALUE;
        }
        var selectedDisplayValue = this.state.displayValue;
        if (!R.isNil(selectedDisplayValue)) {
            return this.tryCalcDisplayValue(this.t(selectedDisplayValue));
        }
        var selectedOptionItem = R.find(function (option) { return option.value === value; }, this.options);
        return R.isNil(selectedOptionItem)
            ? _super.prototype.toDisplayValue.call(this, value)
            : this.tryCalcDisplayValue(R.isNil(selectedOptionItem.label)
                ? selectedOptionItem.value
                : this.t(selectedOptionItem.label));
    };
    Object.defineProperty(BasicSelect.prototype, "options", {
        /**
         * @stable [20.08.2018]
         * @returns {ISelectOptionEntity[]}
         */
        get: function () {
            return this.props.options || [];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BasicSelect.prototype, "menu", {
        /**
         * @stable [20.08.2018]
         * @returns {IMenu}
         */
        get: function () {
            return this.refs.menu;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [20.08.2018]
     * @param {IBasicEvent} event
     */
    BasicSelect.prototype.openMenu = function (event) {
        if (this.menu.isOpen()) {
            return;
        }
        util_1.cancelEvent(event);
        var props = this.props;
        var areOptionsEmpty = R.isNil(props.options);
        if (props.forceReload || areOptionsEmpty) {
            if (props.onEmptyDictionary) {
                this.setState({ needToOpenMenu: true });
                props.onEmptyDictionary();
            }
            else if (!areOptionsEmpty) {
                this.showMenu();
            }
        }
        else {
            this.showMenu();
        }
    };
    /**
     * @stable [20.08.2018]
     */
    BasicSelect.prototype.showMenu = function () {
        var filteredOptions = this.toFilteredOptions();
        if (filteredOptions.length) {
            this.menu.show();
        }
        else {
            BasicSelect.logger.debug('[$BasicSelect][showMenu] The options are empty. The menu does not show.');
        }
    };
    /**
     * @stable [20.08.2018]
     */
    BasicSelect.prototype.hideMenu = function () {
        this.menu.hide();
    };
    BasicSelect.logger = ts_smart_logger_1.LoggerFactory.makeLogger('BasicSelect');
    return BasicSelect;
}(textfield_1.BaseTextField));
exports.BasicSelect = BasicSelect;
//# sourceMappingURL=basic-select.component.js.map