"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var ReactDOM = require("react-dom");
var R = require("ramda");
var env_1 = require("../../env");
var util_1 = require("../../util");
var base_1 = require("../base");
var keyboard_interface_1 = require("./keyboard.interface");
var key_1 = require("./key");
var configurations_definitions_interface_1 = require("../../configurations-definitions.interface");
var Keyboard = /** @class */ (function (_super) {
    __extends(Keyboard, _super);
    /**
     * @stable [03.09.2018]
     * @param {IKeyboardProps} props
     */
    function Keyboard(props) {
        var _this = _super.call(this, props) || this;
        _this.onSelect = _this.onSelect.bind(_this);
        _this.state = { position: _this.currentPosition, mode: 0, useUppercase: false };
        return _this;
    }
    /**
     * @stable [08.05.2018]
     * @returns {React.ReactNode}
     */
    Keyboard.prototype.render = function () {
        var _this = this;
        var props = this.props;
        var state = this.state;
        var keys = props.layout[state.mode];
        var renderToBody = props.renderToBody !== false;
        var el = (React.createElement("div", { ref: this.getSelfRef(), className: util_1.toClassName('rac-keyboard', 'rac-no-user-select', renderToBody && 'rac-keyboard-absolute', props.className) },
            util_1.orNull(renderToBody, function () { return _this.uiFactory.makeIcon({
                key: 'keyboard-close-action-key',
                type: 'ban',
                className: 'rac-keyboard-close',
                onClick: props.onClose,
            }); }),
            keys.map(function (row, index) { return (React.createElement("div", { key: "keyboard-row-" + index, className: 'rac-flex rac-flex-center' }, row.map(function (key, index2) { return (React.createElement(key_1.KeyboardKey, { key: "keyboard-row-column-" + index + "-" + index2, value: key, useUppercase: state.useUppercase, onSelect: _this.onSelect })); }))); })));
        return renderToBody ? ReactDOM.createPortal(el, env_1.ENV.documentBody) : el;
    };
    /**
     * @stable [08.05.2018]
     * @param {KeyboardKeyT} key
     */
    Keyboard.prototype.onSelect = function (key) {
        var nextValue;
        var props = this.props;
        var state = this.state;
        var jEl = this.jField;
        var position = this.currentPosition;
        var chars = jEl.val().split('');
        var keyAsString = key;
        var keyAsObject = key;
        if (util_1.isString(keyAsString) || keyAsObject.type === configurations_definitions_interface_1.KeyboardKeyEnum.SPACE) {
            var keyValue = util_1.isString(keyAsString) ? keyAsString : keyAsObject.value;
            nextValue = R.insert(position, state.useUppercase ? keyValue.toUpperCase() : keyValue, chars);
            props.onChange(nextValue.join(''));
            this.setState({ position: position + 1 });
        }
        else {
            switch (keyAsObject.type) {
                case configurations_definitions_interface_1.KeyboardKeyEnum.UPPERCASE:
                    this.setState({ useUppercase: !state.useUppercase });
                    break;
                case configurations_definitions_interface_1.KeyboardKeyEnum.CHANGE_LAYOUT:
                    this.setState({ mode: this.state.mode === props.layout.length - 1 ? 0 : this.state.mode + 1 });
                    break;
                case configurations_definitions_interface_1.KeyboardKeyEnum.BACKSPACE:
                    if (position > 0) {
                        nextValue = R.remove(position - 1, 1, chars).join('');
                        props.onChange(nextValue);
                        this.setState({ position: position - 1 });
                    }
                    break;
            }
        }
    };
    Object.defineProperty(Keyboard.prototype, "jField", {
        /**
         * TODO dom accessor
         * @returns {IJqInput}
         */
        get: function () {
            return util_1.toJqEl(this.props.field);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Keyboard.prototype, "currentPosition", {
        /**
         * @stable [21.08.2018]
         * @returns {number}
         */
        get: function () {
            var field = this.props.field;
            return util_1.ifNotNilThanValue(field, function () { return field.value.length; }) || 0;
        },
        enumerable: true,
        configurable: true
    });
    Keyboard.defaultProps = {
        layout: [keyboard_interface_1.KEYBOARD_QWERTY_LAYOUT, keyboard_interface_1.KEYBOARD_QWERTY_DIGITAL_LAYOUT],
    };
    return Keyboard;
}(base_1.BaseComponent));
exports.Keyboard = Keyboard;
//# sourceMappingURL=keyboard.component.js.map