"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var R = require("ramda");
var moment = require("moment");
require("moment-timezone");
var di_1 = require("../../di");
var definitions_interface_1 = require("../../definitions.interface");
var util_1 = require("../../util");
var DateConverter = /** @class */ (function () {
    function DateConverter() {
    }
    DateConverter_1 = DateConverter;
    /**
     * @stable [08.01.2018]
     * @param {Date} date1
     * @param {Date} date2
     * @returns {boolean}
     */
    DateConverter.prototype.compare = function (date1, date2) {
        return !R.isNil(date1) && !R.isNil(date2) && date1.getTime() === date2.getTime();
    };
    /**
     * @stable [11.08.2018]
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @param {string} outputFormat
     * @returns {string}
     */
    DateConverter.prototype.format = function (date, inputFormat, outputFormat) {
        if (R.isNil(date) || R.isEmpty(date)) {
            return '';
        }
        else {
            var momentDate = this.toMomentDate(date, inputFormat);
            return momentDate.isValid()
                ? momentDate.format(outputFormat)
                : String(date);
        }
    };
    /**
     * @stable [09.11.2018]
     * @param {DateTimeLikeTypeT} date
     * @param {string} outputFormat
     * @returns {string}
     */
    DateConverter.prototype.fromDateTimeToArbitraryFormat = function (date, outputFormat) {
        return this.format(date, this.dateTimeFormat, outputFormat);
    };
    /**
     * @stable [09.11.2018]
     * @param {DateTimeLikeTypeT} date [Example: 2018-04-07T20:54:45+03:00]
     * @returns {string} [Example: Apr 07 08:54 PM]
     */
    DateConverter.prototype.fromDateTimeToPstDateTime = function (date) {
        if (date === void 0) { date = this.currentDate; }
        return this.fromDateTimeToArbitraryFormat(date, this.pstDateTimeFormat);
    };
    /**
     * @stable [09.11.2018]
     * @param {DateTimeLikeTypeT} date [Example: 2018-11-30T03:00:00+03:00]
     * @returns {string} [Example: Nov 30]
     */
    DateConverter.prototype.fromDateTimeToPstDate = function (date) {
        if (date === void 0) { date = this.currentDate; }
        return this.fromDateTimeToArbitraryFormat(date, this.dateTimeSettings.pstDateFormat);
    };
    /**
     * @stable [08.01.2019]
     * @param {DateTimeLikeTypeT} date
     * @returns {string}
     */
    DateConverter.prototype.fromDateTimeToPstTime = function (date) {
        if (date === void 0) { date = this.currentDate; }
        return this.fromDateTimeToArbitraryFormat(date, this.dateTimeSettings.pstTimeFormat);
    };
    /**
     * @stable [09.11.2018]
     * @param {DateTimeLikeTypeT} date [Example: 2018-04-07T20:54:45+03:00]
     * @returns {string} [Example: 2018-04-07]
     */
    DateConverter.prototype.fromDateTimeToDate = function (date) {
        return this.fromDateTimeToArbitraryFormat(date, this.dateFormat);
    };
    /**
     * @stable [09.11.2018]
     * @param {DateTimeLikeTypeT} date [Example: 2018-04-07T20:54:45+03:00]
     * @returns {string} [Example: 20:54:45]
     */
    DateConverter.prototype.fromDateTimeToTime = function (date) {
        return this.fromDateTimeToArbitraryFormat(date, this.timeFormat);
    };
    /**
     * @stable [23.11.2018]
     * @param {DateTimeLikeTypeT} date [Example: Wed Oct 24 2018 04:07:41 GMT+0300 (RTZ 2 (зима))]
     * @returns {string} [Example: 2018-10-24T04:07:41+03:00]
     */
    DateConverter.prototype.fromDateTimeToDateTime = function (date) {
        return this.fromDateTimeToArbitraryFormat(date, this.dateTimeFormat);
    };
    /**
     * @stable [24.11.2018]
     * @param {string} date [Example: 2018-11-24]
     * @param {string} time [Example: 03:47:17]
     * @returns {string} [Example: 2018-11-24T03:47:17+03:00]
     */
    DateConverter.prototype.fromUiDateTimeToDateTime = function (date, time) {
        return this.format(this.combine(date, time), this.uiDateTimeFormat, this.dateTimeFormat);
    };
    /**
     * @stable [24.11.2018]
     * @param {string} date [Example: 2018-11-24]
     * @param {string} time [Example: 03:47:17]
     * @returns {string} [Example: Nov 24]
     */
    DateConverter.prototype.fromUiDateTimeToPstDate = function (date, time) {
        return this.fromDateTimeToPstDate(this.fromUiDateTimeToDateTime(date, time));
    };
    /**
     * @stable [09.11.2018]
     * @param {TEntity} entity
     * @param {string} dateFieldName
     * @param {string} timeFieldName
     * @param {(entity: TEntity) => string} dateResolver
     * @returns {IKeyValue}
     */
    DateConverter.prototype.splitToDateTimeFields = function (entity, dateFieldName, timeFieldName, dateResolver) {
        var _this = this;
        var _a;
        var date = dateResolver(entity);
        return util_1.defValuesFilter((_a = {},
            _a[timeFieldName] = util_1.orUndef(date, function () { return _this.fromDateTimeToTime(date); }),
            _a[dateFieldName] = util_1.orUndef(date, function () { return _this.fromDateTimeToDate(date); }),
            _a));
    };
    /**
     * @stable [07.01.2019]
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {DateTimeLikeTypeT}
     */
    DateConverter.prototype.tryConvertToDate = function (date, inputFormat) {
        if (inputFormat === void 0) { inputFormat = this.dateTimeSettings.uiDateFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        return momentDate.isValid() ? momentDate.toDate() : date;
    };
    /**
     * @stable [07.01.2019]
     * @param {moment.DurationInputArg2} unit
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.tryAddXDurationAsMomentDate = function (unit, duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.dateTimeSettings.uiDateFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        return util_1.orNull(momentDate.isValid(), function () { return momentDate.add(duration, unit); });
    };
    /**
     * @stable [07.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.tryAddXDaysAsMomentDate = function (duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.dateTimeSettings.uiDateFormat; }
        return this.tryAddXDurationAsMomentDate('days', duration, date, inputFormat);
    };
    /**
     * @stable [07.01.2019]
     * @param {moment.DurationInputArg2} unit
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryAddXDuration = function (unit, duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.dateTimeSettings.uiDateFormat; }
        return util_1.ifNotNilThanValue(this.tryAddXDurationAsMomentDate(unit, duration, date, inputFormat), function (value) { return value.toDate(); });
    };
    /**
     * @stable [07.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryAddXDays = function (duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.dateTimeSettings.uiDateFormat; }
        return this.tryAddXDuration('days', duration, date, inputFormat);
    };
    /**
     * @stable [07.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryAddXMonths = function (duration, date, inputFormat) {
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.dateTimeSettings.uiDateFormat; }
        return this.tryAddXDuration('months', duration, date, inputFormat);
    };
    /**
     * @stable [08.01.2018]
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    /**
     * @stable [08.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.tryGetFirstDayOfMonthAsMomentDate = function (duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.dateTimeSettings.uiDateFormat; }
        return util_1.ifNotNilThanValue(this.tryAddXDurationAsMomentDate('months', duration, date, inputFormat), function (value) { return value.startOf('month'); });
    };
    /**
     * @stable [08.01.2019]
     * @param {moment.DurationInputArg1} duration
     * @param {DateTimeLikeTypeT} date
     * @param {string | undefined} inputFormat
     * @returns {Date}
     */
    DateConverter.prototype.tryGetFirstDayOfMonth = function (duration, date, inputFormat) {
        if (duration === void 0) { duration = 0; }
        if (date === void 0) { date = this.currentDate; }
        if (inputFormat === void 0) { inputFormat = this.dateTimeSettings.uiDateFormat; }
        return util_1.ifNotNilThanValue(this.tryGetFirstDayOfMonthAsMomentDate(duration, date, inputFormat), function (value) { return value.toDate(); });
    };
    DateConverter.prototype.fromDateToUiDate = function (date) {
        return this.format(this.shrinkDate(date), /* Need to clear a timezone */ this.dateFormat, this.uiDateFormat);
    };
    DateConverter.prototype.fromDateTimeToUiDate = function (date) {
        return this.fromDateTimeToArbitraryFormat(date, this.uiDateFormat);
    };
    DateConverter.prototype.fromDateTimeToUiDateTime = function (date) {
        return this.fromDateTimeToArbitraryFormat(date, this.uiDateTimeFormat);
    };
    /**
     * @test
     * @param {string} startUiDate [2018-01-31]
     * @param {string} startUiTime [21:58:59]
     * @returns {string} The formatted value [2018-01-31T21:58:59-08:00]
     */
    DateConverter.prototype.fromStartUiDateTimeToDateTime = function (startUiDate, startUiTime) {
        if (startUiTime === void 0) { startUiTime = definitions_interface_1.DEFAULT_TIME_FROM; }
        return this.fromUiDateTimeToDateTime(startUiDate, startUiTime);
    };
    /**
     * @test
     * @param {string} endUiDate [2018-01-31]
     * @param {string} endUiTime [23:59:59]
     * @returns {string} The formatted value [2018-01-31T23:59:59-08:00]
     */
    DateConverter.prototype.fromEndUiDateTimeToDateTime = function (endUiDate, endUiTime) {
        if (endUiTime === void 0) { endUiTime = definitions_interface_1.DEFAULT_TIME_TO; }
        return this.fromUiDateTimeToDateTime(endUiDate, endUiTime);
    };
    /**
     * @test
     * @returns {string}
     */
    DateConverter.prototype.from30DaysAgoUiDateTimeToDateTime = function () {
        return this.fromUiDateTimeToDateTime(this.fromDateToUiDate(this.get30DaysAgo()), definitions_interface_1.DEFAULT_TIME_FROM);
    };
    /**
     * @stable [02.06.2018]
     * @returns {Date}
     */
    DateConverter.prototype.get30DaysAgo = function () {
        return this.getXDaysAgo(30);
    };
    /**
     * @stable [02.06.2018]
     * @param {number} days
     * @returns {Date}
     */
    DateConverter.prototype.getXDaysLater = function (days) {
        return this.getCurrentMomentDate().add(days, 'days').toDate();
    };
    /**
     * @stable [02.06.2018]
     * @param {number} days
     * @returns {Date}
     */
    DateConverter.prototype.getXDaysAgo = function (days) {
        return this.getCurrentMomentDate().subtract(days, 'days').toDate();
    };
    DateConverter.prototype.addXDays = function (date, days) {
        return this.toMomentDate(date).add(days, 'days').toDate();
    };
    /**
     * @returns {Date}
     */
    DateConverter.prototype.getCurrentDate = function () {
        return this.getCurrentMomentDate().toDate();
    };
    DateConverter.prototype.formatDate = function (date, outputFormat) {
        return this.format(date, this.dateFormat, outputFormat);
    };
    DateConverter.prototype.formatDateTimeFromDateTime = function (date) {
        return this.formatDateTime(date, this.dateTimeFormat);
    };
    DateConverter.prototype.formatTimeFromDateTime = function (date) {
        return this.formatDateTime(date, this.timeFormat);
    };
    DateConverter.prototype.formatDateTime = function (date, outputFormat) {
        return this.format(date, this.dateTimeFormat, outputFormat);
    };
    DateConverter.prototype.formatPSTDate = function (date) {
        if (date === void 0) { date = new Date(); }
        return this.formatDate(date, this.dateTimeSettings.pstDateFormat);
    };
    DateConverter.prototype.appendToDate = function (date, data, inputFormat) {
        if (inputFormat === void 0) { inputFormat = this.dateFormat; }
        var momentDate = this.toMomentDate(date, inputFormat);
        data.forEach(function (item) {
            momentDate = momentDate.add(item[0], item[1]);
        });
        return momentDate.toDate();
    };
    DateConverter.prototype.getLocalizedMonth = function (index) {
        return DateConverter_1.MONTHS[index];
    };
    DateConverter.prototype.getLocalizedMonthShort = function (index) {
        return DateConverter_1.MONTHS_SHORT[index];
    };
    DateConverter.prototype.getLocalizedWeekday = function (index) {
        return DateConverter_1.WEEKDAYS[index];
    };
    DateConverter.prototype.getLocalizedWeekdayShort = function (index) {
        return DateConverter_1.WEEKDAYS_SHORT[index];
    };
    /**
     * @stable [25.08.2018]
     * @param {number} monthsAgo
     * @returns {Date}
     */
    DateConverter.prototype.getLastDayOfMonth = function (monthsAgo) {
        if (monthsAgo === void 0) { monthsAgo = 1; }
        return moment().subtract(monthsAgo, 'months').endOf('month').toDate();
    };
    /**
     * @stable [25.08.2018]
     * @param {number} monthsAgo
     * @returns {Date}
     */
    DateConverter.prototype.getFirstDayOfMonth = function (monthsAgo) {
        if (monthsAgo === void 0) { monthsAgo = 0; }
        return moment().subtract(monthsAgo, 'months').startOf('month').toDate();
    };
    /**
     * @stable [24.11.2018]
     * @param {string} dateAsString
     * @param {string} timeAsString
     * @returns {string}
     */
    DateConverter.prototype.combine = function (dateAsString, timeAsString) {
        return [dateAsString, timeAsString].join(' ');
    };
    /**
     * @stable [25.08.2018]
     * @param {DateTimeLikeTypeT} dateTime
     * @param {string} separator
     * @returns {DateTimeLikeTypeT}
     */
    DateConverter.prototype.shrinkDate = function (dateTime, separator) {
        if (separator === void 0) { separator = 'T'; }
        return util_1.isString(dateTime) ? dateTime.split(separator)[0] : dateTime;
    };
    /**
     * @stable [11.08.2018]
     * @param {DateTimeLikeTypeT} date
     * @param {string} inputFormat
     * @returns {moment.Moment}
     */
    DateConverter.prototype.toMomentDate = function (date, inputFormat) {
        var momentDate = date instanceof Date
            ? moment(date)
            : moment(date, inputFormat, true);
        var zone = this.timeZone;
        return zone ? moment.tz(date, zone) : momentDate;
    };
    DateConverter.prototype.getCurrentMomentDate = function () {
        return this.toMomentDate(this.currentDate);
    };
    Object.defineProperty(DateConverter.prototype, "timeZone", {
        /**
         * @stable [11.08.2018]
         * @returns {string}
         */
        get: function () {
            return this.dateTimeSettings.timeZone;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "currentDate", {
        get: function () {
            return this.dateTimeSettings.currentDate;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "dateTimeFormat", {
        get: function () {
            return this.dateTimeSettings.dateTimeFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "dateFormat", {
        get: function () {
            return this.dateTimeSettings.dateFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "timeFormat", {
        get: function () {
            return this.dateTimeSettings.timeFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "uiDateFormat", {
        get: function () {
            return this.dateTimeSettings.uiDateFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "uiTimeFormat", {
        get: function () {
            return this.dateTimeSettings.uiTimeFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "uiDateTimeFormat", {
        get: function () {
            return this.combine(this.uiDateFormat, this.uiTimeFormat);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "pstDateTimeFormat", {
        get: function () {
            return this.combine(this.dateTimeSettings.pstDateFormat, this.dateTimeSettings.pstTimeFormat);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateConverter.prototype, "dateTimeSettings", {
        get: function () {
            return this.applicationSettings.dateTime || {};
        },
        enumerable: true,
        configurable: true
    });
    var DateConverter_1;
    DateConverter.MONTHS = moment.months();
    DateConverter.MONTHS_SHORT = moment.monthsShort();
    DateConverter.WEEKDAYS_SHORT = moment.weekdaysShort()
        .slice(1, 7)
        .concat(moment.weekdaysShort()[0]);
    DateConverter.WEEKDAYS = moment.weekdays()
        .slice(1, 7)
        .concat(moment.weekdays()[0]);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.Settings),
        __metadata("design:type", Object)
    ], DateConverter.prototype, "applicationSettings", void 0);
    DateConverter = DateConverter_1 = __decorate([
        inversify_1.injectable()
    ], DateConverter);
    return DateConverter;
}());
exports.DateConverter = DateConverter;
//# sourceMappingURL=date-converter.service.js.map