"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var R = require("ramda");
var URI = require("urijs");
var ts_smart_logger_1 = require("ts-smart-logger");
var di_1 = require("../di");
var util_1 = require("../util");
var transport_interface_1 = require("./transport.interface");
var TransportFactory = /** @class */ (function () {
    function TransportFactory() {
        this.id = 0;
        this.operationsMap = new Map();
    }
    TransportFactory_1 = TransportFactory;
    TransportFactory.prototype.request = function (req) {
        var _this = this;
        var cancelToken;
        var operationId = req.operation && req.operation.id;
        if (operationId) {
            cancelToken = this.requestFactory.cancelToken;
            if (cancelToken) {
                this.operationsMap.set(operationId, cancelToken);
            }
        }
        var headers = {};
        var isBinaryData = util_1.isDef(req.blob);
        if (isBinaryData) {
            headers = {
                'Content-Type': 'application/octet-stream',
            };
        }
        var uri0 = new URI(req.url || [
            isBinaryData ? this.settings.binaryUrl : this.settings.apiUrl,
            req.path || ''
        ].join('') // URI's segment works incorrectly with a UUID (uri0.segment(req.path))
        );
        if (req.noCache !== true) {
            uri0.addSearch('_dc', Date.now());
        }
        return this.requestFactory.request(util_1.defValuesFilter({
            headers: headers,
            url: uri0.valueOf(),
            method: req.method || 'POST',
            data: this.getRequestData(req),
            withCredentials: true,
            cancelToken: cancelToken && cancelToken.token,
        }))
            .then(function (res) {
            TransportFactory_1.logger.debug("[$TransportFactory][request] Data have been loaded. Result: ", res);
            _this.clearOperation(operationId);
            return _this.getResponseData(res);
        }, function (err) {
            TransportFactory_1.logger.debug("[$TransportFactory][request] An error occurred during a request. Error: " + err);
            _this.clearOperation(operationId);
            if (err
                // There is no other way to determine a cancel action
                && err.message === TransportFactory_1.CANCEL_MESSAGE) {
                return Promise.reject(transport_interface_1.TRANSPORT_REQUEST_CANCEL_REASON);
            }
            throw err;
        });
    };
    TransportFactory.prototype.cancelRequest = function (operationId) {
        var cancelToken = this.operationsMap.get(operationId);
        if (cancelToken) {
            cancelToken.cancel(TransportFactory_1.CANCEL_MESSAGE);
            this.clearOperation(operationId);
        }
        else {
            TransportFactory_1.logger.warn("[$TransportFactory][cancelRequest] The cancel token has not been found according to " + operationId);
        }
    };
    TransportFactory.prototype.toRequestParams = function (req) {
        var _this = this;
        return util_1.defValuesFilter({
            id: this.id++,
            name: req.name,
            params: util_1.orUndef(req.params, function () { return util_1.defValuesFilter(req.params); }),
            auth: util_1.orUndef(!req.notApplyAuth, function () { return _this.tokenAccessor.token; }),
        });
    };
    TransportFactory.prototype.getResponseData = function (res) {
        return res;
    };
    TransportFactory.prototype.getRequestData = function (req) {
        return req.blob || this.toRequestParams(req);
    };
    TransportFactory.prototype.clearOperation = function (operationId) {
        if (!R.isNil(operationId)) {
            this.operationsMap.delete(operationId);
        }
    };
    var TransportFactory_1;
    TransportFactory.logger = ts_smart_logger_1.LoggerFactory.makeLogger('TransportFactory');
    TransportFactory.CANCEL_MESSAGE = 'The operation has been canceled by the user.';
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.Settings),
        __metadata("design:type", Object)
    ], TransportFactory.prototype, "settings", void 0);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.TransportRequestFactory),
        __metadata("design:type", Object)
    ], TransportFactory.prototype, "requestFactory", void 0);
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.TransportTokenAccessor),
        __metadata("design:type", Object)
    ], TransportFactory.prototype, "tokenAccessor", void 0);
    TransportFactory = TransportFactory_1 = __decorate([
        inversify_1.injectable()
    ], TransportFactory);
    return TransportFactory;
}());
exports.TransportFactory = TransportFactory;
//# sourceMappingURL=transport.factory.js.map