"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
/**
 * @stable [01.08.2018]
 * @param {IPlaceEntity} placeEntity
 * @returns {string}
 */
exports.toAddress = function (placeEntity) {
    if (!R.isNil(placeEntity)) {
        return [
            [placeEntity.streetNumber, placeEntity.street].filter(function (v) { return !!v; }).join(' '),
            placeEntity.city,
            placeEntity.region,
            placeEntity.country
        ].filter(function (v) { return !!v; }).join(', ');
    }
    return '';
};
/**
 * @stable [01.08.2018]
 * @param {google.maps.GeocoderAddressComponent} cmp
 * @param {(cmp: google.maps.GeocoderAddressComponent) => TType} extractor
 * @returns {TType}
 */
exports.toPlaceDescription = function (cmp, extractor) {
    return cmp && extractor(cmp);
};
/**
 * @stable [04.08.2018]
 * @param {google.maps.GeocoderResult | google.maps.places.PlaceResult} place
 * @returns {IPlaceEntity}
 */
exports.toPlace = function (place) {
    if (!Array.isArray(place.address_components)) {
        return null;
    }
    return {
        country: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('country'); }), function (cmp) { return cmp.long_name; }),
        region: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('administrative_area_level_1'); }), function (cmp) { return cmp.long_name; }),
        area: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('administrative_area_level_2'); }), function (cmp) { return cmp.long_name; }),
        city: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('locality'); }), function (cmp) { return cmp.long_name; }),
        street: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('route'); }), function (cmp) { return cmp.long_name; }),
        streetNumber: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('street_number'); }), function (cmp) { return cmp.long_name; }),
        zipCode: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('postal_code'); }), function (cmp) { return cmp.long_name; }),
    };
};
//# sourceMappingURL=place.js.map